using System;
using System.Data;
using System.Data.Common;
using System.IO;
using System.Web;

namespace Vevo.Shared.DataAccess.Tester
{
    public class DatabaseTester
    {
        private string _connectionString;
        private string _providerName;
        private DataAccess _dataAccess;

        private static string GetFullPathNameFromDBFileName( string relativeFileName, string appLocalPath )
        {
            relativeFileName = relativeFileName.Replace( '/', '\\' ); // replace / with \
            if (relativeFileName.StartsWith( "~\\" ))
                relativeFileName = relativeFileName.Substring( 2 );
            else if (relativeFileName.StartsWith( "\\" ))
                relativeFileName = relativeFileName.Substring( 1 );

            return Path.Combine( appLocalPath, relativeFileName );
        }

        public DatabaseTester( string connectionString, string providerName )
        {
            _connectionString = connectionString;
            _providerName = providerName;
            _dataAccess = new DataAccess( _connectionString, _providerName );
        }

        public DatabaseTester( string connectionPrefix, string connectionString, string providerName, string appLocalPath )
        {
            _connectionString = connectionPrefix + GetFullPathNameFromDBFileName( connectionString, appLocalPath );
            _providerName = providerName;
            _dataAccess = new DataAccess( _connectionString, _providerName );
        }


        public bool TestConnection( out string message )
        {
            bool success = true;
            message = string.Empty;

            DbCommand command = _dataAccess.CreateTextCommand();
            try
            {
                command.Connection.Open();
            }
            catch (Exception ex)
            {
                message =
                    "Connection String: " + command.Connection.ConnectionString + "\n\n" +
                    ex.ToString();
                success = false;
            }
            finally
            {
                command.Connection.Close();
            }

            return success;
        }

        public bool TestSelectCommand( string tableName, out string message )
        {
            bool success = true;
            message = string.Empty;

            string commandString = "SELECT * FROM " + tableName + "; ";
            try
            {
                DataTable table = _dataAccess.ExecuteSelect( commandString );
            }
            catch (Exception ex)
            {
                message = "Command String: " + commandString + "\n\n" + ex.ToString();
                success = false;
            }

            return success;
        }

        public bool TestCreateTable( string tableName, out string message )
        {
            bool success = true;
            message = string.Empty;

            string commandString = "INSERT INTO [" + tableName + "] (PageID, CultureID, KeyName, TextData) " +
                "VALUES (2999999, 2999999, 'asdf', 'eoutirer'); ";
            string cleanUpString = "DELETE FROM [" + tableName + "] WHERE PageID = 2999999;";

            try
            {
                _dataAccess.ExecuteNonQuery( commandString );

                // If success, clean up test table
                try
                {
                    _dataAccess.ExecuteNonQuery( cleanUpString );
                }
                catch
                {
                }
            }
            catch (Exception ex)
            {
                message = "CommandString: " + commandString + "\n\n" + ex.ToString();
                success = false;
            }

            return success;
        }

    }
}
