using System;
using System.Text.RegularExpressions;
using Vevo.UrlModuleLib.Configuration;

namespace Vevo.UrlModuleLib
{
    public class RewriteUrlProcessor : UrlProcessor
    {
        private string _applicationPath;
        private string _mobileApplicationPath;
        private UrlConfiguration _config;

        private string AppendQuery( string url, string query )
        {
            string result = url;

            if ( !String.IsNullOrEmpty( query ) )
            {
                if ( result.IndexOf( '?' ) == -1 )
                    result += query;
                else
                    result += '&' + query.TrimStart( new char[] { '?' } );
            }

            return result;
        }

        private string EncodeUrl( string url )
        {
            return System.Web.HttpContext.Current.Server.UrlEncode( url );
        }

        public RewriteUrlProcessor()
        {
        }

        public override void SetUp( string applicationPath, string mobileApplicationPath, UrlConfiguration config )
        {
            _applicationPath = applicationPath;
            _mobileApplicationPath = mobileApplicationPath;
            _config = config;
        }

        public override string ProcessIncomingUrl( string originalPath, string originalQueryString, bool isMobile, out string data )
        {
            string sendTo = originalPath;
            data = string.Empty;
            bool matched = false;
            int i = 0;

            while ( !matched && i < _config.IncomingRules.Count )
            {
                string lookFor = String.Empty;

                if ( isMobile )
                {
                    lookFor = "^" +
                    UrlUtilities.ResolveUrl( _applicationPath, _config.IncomingRules[ i ].MobileLookFor ) +
                    "$";
                }
                else
                {
                    lookFor = "^" +
                    UrlUtilities.ResolveUrl( _applicationPath, _config.IncomingRules[ i ].LookFor ) +
                    "$";
                }

                Regex re = new Regex( lookFor, RegexOptions.IgnoreCase );

                if ( re.IsMatch( originalPath ) )
                {
                    matched = true;
                    string newUrl = String.Empty;
                    GroupCollection regGroup = re.Match( originalPath ).Groups;

                    if ( isMobile )
                    {
                        newUrl = _config.IncomingRules[ i ].MobileSendTo.Replace( "{Name}", EncodeUrl( regGroup[ "Name" ].Value ) );
                        data = re.Replace( originalPath, _config.IncomingRules[ i ].MobileOutData );
                    }
                    else
                    {
                        newUrl = _config.IncomingRules[ i ].SendTo.Replace( "{Name}", EncodeUrl( regGroup[ "Name" ].Value ) );
                        data = re.Replace( originalPath, _config.IncomingRules[ i ].OutData );
                    }

                    sendTo = UrlUtilities.ResolveUrl( _applicationPath, newUrl );
                    data = UrlUtilities.ResolveUrl( _applicationPath, data );
                }

                i++;
            }

            sendTo = AppendQuery( sendTo, originalQueryString );

            return sendTo;
        }

        /* Remove Function Utilities.RemoveDiacritics() From change Url Name */

        public override string ResolveOutgoingCategoryUrl( string CategoryID, string CategoryName )
        {
            string url = _config.OutgoingRules.CategoryRule;
            return url.Replace( "{CategoryName}", CategoryName.ToLower() ).Replace( "{CategoryID}", CategoryID );
        }

        public override string ResolveOutgoingDepartmentUrl( string DepartmentID, string DepartmentName )
        {
            string url = _config.OutgoingRules.DepartmentRule;
            return url.Replace( "{DepartmentName}", DepartmentName.ToLower() ).Replace( "{DepartmentID}", DepartmentID );
        }

        public override string ResolveOutgoingManufacturerUrl( string ManufacturerID, string ManufacturerUrlName )
        {
            string url = _config.OutgoingRules.ManufacturerRule;
            return url.Replace( "{ManufacturerName}", ManufacturerUrlName ).Replace( "{ManufacturerID}", ManufacturerID );
        }
		
        public override string ResolveOutgoingProductUrl( string ProductID, string ProductName )
        {
            string url = _config.OutgoingRules.ProductRule;
            return url.Replace( "{ProductName}", ProductName.ToLower() ).Replace( "{ProductID}", ProductID );
        }

        public override string ResolveOutgoingTellFriendUrl( string productUrlName )
        {
            return string.Format(
                _config.OutgoingRules.TellFriendRule,
                productUrlName );
        }

        public override string ResolveOutgoingCustomerReviewUrl( string productUrlName )
        {
            return string.Format(
                _config.OutgoingRules.CustomerReviewRule,
                 productUrlName );
        }

        public override string ResolveOutgoingContentUrl( string ContentID, string ContentName )
        {
            string url = _config.OutgoingRules.ContentRule;
            return url.Replace( "{ContentName}", ContentName.ToLower() ).Replace( "{ContentID}", ContentID );
        }

        public override string ResolveOutgoingContentMenuUrl( string ContentMenuItemNameID, string ContentMenuItemName )
        {
            string url = _config.OutgoingRules.ContentMenuRule;
            return url.Replace( "{ContentMenuItemName}", ContentMenuItemName )
                .Replace( "{ContentMenuItemNameID}", ContentMenuItemNameID );
        }
        public override string ResolveOutgoingNewsUrl( string NewsID, string NewsName )
        {
            string url = _config.OutgoingRules.NewsRule;
            return url.Replace( "{NewsName}", NewsName.ToLower() ).Replace( "{NewsID}", NewsID );
        }
        public override string ResolveOutgoingBlogUrl( string BlogID, string BlogName )
        {
            string url = _config.OutgoingRules.BlogRule;
            return url.Replace( "{BlogName}", BlogName.ToLower() ).Replace( "{BlogID}", BlogID );
        }

        public override string ResolveOutgoingBlogListUrl( string BlogListName )
        {
            string url = _config.OutgoingRules.BlogListRule;
            return url.Replace( "{BlogListName}", BlogListName.ToLower() );
        }

        public override string ResolveOutgoingPromotionUrl( string PromotionID, string PromotionName )
        {
            string url = _config.OutgoingRules.PromotionRule;
            return url.Replace( "{PromoName}", PromotionName.ToLower() ).Replace( "{PromoID}", PromotionID );
        }
    }
}
