using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;

using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CurrencyRepository : ICurrencyRepository
    {
        #region Private

        private const string RepositoryName = "CurrencyRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetCodeByNameSignature = "GetCodeByName";
        private const string GetByEnabledSignature = "GetByEnabled";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string SearchCurrencySignature = "SearchCurrency";
        private const string TableName = "Currency";
        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );
        private CurrencyRepositoryNoCache _repository = new CurrencyRepositoryNoCache();

        #endregion

        #region Constructors

        public CurrencyRepository()
        {
        }

        public CurrencyRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public Currency Save( Currency currency, string newCurrencyCode )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Currency newCurrency = _repository.Save( currency, newCurrencyCode );
            _dataCache.ClearRepositoryCache();
            return newCurrency;
        }

        public Currency GetOne( string currencyCode )
        {
            Currency result = (Currency) _dataCache.GetValue( GetOneSignature, _cacheDependency, currencyCode );

            if (result == null)
            {
                result = _repository.GetOne( currencyCode );
                _dataCache.SetValue( GetOneSignature, result, currencyCode );
            }

            return result;
        }

        public void Delete( string currencyCode )
        {
            _repository.Delete( currencyCode );
            _dataCache.ClearRepositoryCache();
        }

        public string GetCodeByName( string name )
        {
            object result = _dataCache.GetValue( GetCodeByNameSignature, _cacheDependency, name );

            if (result == null)
            {
                result = _repository.GetCodeByName( name );
                _dataCache.SetValue( GetCodeByNameSignature, result, name );
            }

            return result.ToString();
        }

        public IList<Currency> GetByEnabled( BoolFilter enabled )
        {
            IList<Currency> result = (IList<Currency>) _dataCache.GetValue( GetByEnabledSignature, _cacheDependency, enabled.ToString() );

            if (result == null)
            {
                result = _repository.GetByEnabled( enabled );
                _dataCache.SetValue( GetByEnabledSignature, result, enabled.ToString() );
            }

            return result;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }

            return list;
        }

        public IList<Currency> SearchCurrency(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchCurrencySignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchCurrency(
                    sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchCurrencySignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Currency>) result.First;
        }


        #endregion
    }
}
