using System;
using System.Collections.Generic;
using System.Data;
using System.Text;

using Vevo.Shared.DataAccess;
using System.Collections;

namespace Vevo.Data.Implementation
{
    internal class ProductCategoryAccess
    {
        #region Private

        private IList<String> ConvertToCategoryIDs( DataTable table )
        {
            IList<String> list = new List<String>();
            foreach (DataRow row in table.Rows)
            {
                list.Add( row["CategoryID"].ToString() );
            }

            return list;
        }

        private Dictionary<string, string> SetProductCategorySortOrder( string productID, IList<String> categoryIDs )
        {
            Dictionary<string, string> categoryList = new Dictionary<string, string>();
            DataTable table = DataAccess.ExecuteSelect(
                  "SELECT * " +
                  "FROM ProductCategory " +
                  "WHERE ProductID = @ProductID; ",
                  DataAccess.CreateParameterString( productID ) );
            foreach (DataRow row in table.Rows)
            {
                if (categoryIDs.Contains( row["CategoryID"].ToString() ))
                {
                    if (String.IsNullOrEmpty( row["CategoryID"].ToString() ))
                    {
                        row["CategoryID"] = "0";
                    }
                    categoryList.Add( row["CategoryID"].ToString(), row["SortOrder"].ToString() );
                }
            }
            return categoryList;
        }
        #endregion


        #region Public Methods

        public void SaveAll( string productID, IList<String> categoryIDs )
        {
            Dictionary<string, string> categoryList = SetProductCategorySortOrder( productID, categoryIDs );
            DeleteByProductID( productID );
            Create( productID, categoryIDs, categoryList );
        }

        public void SaveAll( string productID, IList<String> categoryIDs, ArrayList categoryVisibleIDList, out bool isParentVisible )
        {
            Dictionary<string, string> categoryList = SetProductCategorySortOrder( productID, categoryIDs ); 
            DeleteByProductID( productID );
            Create( productID, categoryIDs, categoryVisibleIDList, out isParentVisible, categoryList );
        }

        public void Create( string productID, IList<String> categoryIDs, Dictionary<string, string> categoryList )
        {
            string sortOrder = String.Empty;
            foreach (string categoryID in categoryIDs)
            {
                if (categoryList.ContainsKey( categoryID ))
                {
                    sortOrder = categoryList[categoryID];
                }
                else
                {
                    sortOrder = "0";
                }
                int result = DataAccess.ExecuteNonQuery(
                    "INSERT INTO ProductCategory (ProductID, CategoryID, SortOrder) " +
                    "VALUES (@ProductID, @CategoryID, @SortOrder); ",
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( categoryID ),
                DataAccess.CreateParameterString( sortOrder ) );

                if (result != 1)
                {
                    Exception exception = new ApplicationException(
                            "Failed creating product. " +
                        "Inserting into ProductCategory returns " + result.ToString() );
                    DataAccessHelper.LogError( exception );
                    throw exception;
                }
            }
        }

        private bool EvaluateParentVisibility( string categoryID, ArrayList categoryVisibleIDList )
        {
            // This product is visible if any of the parent categories is visible.
            if (categoryVisibleIDList.Contains( categoryID ))
                return true;

            return false;
        }

        public void Create( string productID, IList<String> categoryIDs, ArrayList categoryVisibleIDList, out bool isParentVisible, Dictionary<string, string> categoryList )
        {
            isParentVisible = false;
            string sortOrder = String.Empty;

            foreach (string categoryID in categoryIDs)
            {
                if (categoryList.ContainsKey( categoryID ))
                {
                    sortOrder = categoryList[categoryID];
                }
                else
                {
                    sortOrder = "0";
                }
                int result = DataAccess.ExecuteNonQuery(
                    "INSERT INTO ProductCategory (ProductID, CategoryID, SortOrder) " +
                    "VALUES (@ProductID, @CategoryID, @SortOrder); ",
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( categoryID ),
                DataAccess.CreateParameterString( sortOrder ) );

                if (result != 1)
                {
                    Exception exception = new ApplicationException(
                            "Failed creating product. " +
                        "Inserting into ProductCategory returns " + result.ToString() );
                    DataAccessHelper.LogError( exception );
                    throw exception;
                }

                if (EvaluateParentVisibility( categoryID, categoryVisibleIDList ))
                    isParentVisible = true;
            }
        }

        public void DeleteByProductID( string productID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ProductCategory " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productID ) );
        }

        public IList<String> GetByProductID( string productID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT CategoryID " +
                "FROM ProductCategory " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productID ) );

            return ConvertToCategoryIDs( table );
        }

        #endregion

    }
}
