using System;
using System.Collections;
using System.Data;
using System.Data.Common;
using System.Configuration;
using System.Text;
using Vevo;
using Vevo.Domain;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;

namespace Vevo.Data.Implementation
{
    internal class ProductGiftCertificateAccess
    {
        private const string ExpireTypeNoneString = "No";
        private const string ExpireTypeFixedDateString = "FixedDate";
        private const string ExpireTypeRollingDateString = "RollingDate";


        private const string AllProductGiftCertificateFields =
            "ProductID, GiftAmount, ExpireType, FixedExpireDate, NumberOfDays, IsElectronic ";

        private string ConvertExpireTypeToString( GiftCertificateProduct.ExpireTypeEnum expireType )
        {
            switch (expireType)
            {
                case GiftCertificateProduct.ExpireTypeEnum.None: return ExpireTypeNoneString;
                case GiftCertificateProduct.ExpireTypeEnum.FixedDate: return ExpireTypeFixedDateString;
                case GiftCertificateProduct.ExpireTypeEnum.RollingDate: return ExpireTypeRollingDateString;
                default:
                    throw new DataException(
                        "Unknown Gift Certificate Expire Type Enum: " + expireType.ToString() );
            }
        }

        public Product Create( Product product )
        {
            GiftCertificateProduct gift = (GiftCertificateProduct) product;

            DataAccess.ExecuteNonQuery(
                "INSERT INTO ProductGiftCertificate " +
                " (" + AllProductGiftCertificateFields + ") " +
                "VALUES (@ProductID, @GiftAmount, @ExpireType, @FixedExpireDate, @NumberOfDays, @IsElectronic);",
                DataAccess.CreateParameterString( gift.ProductID ),
                DataAccess.CreateParameterDecimal( gift.GiftAmount ),
                DataAccess.CreateParameterString( ConvertExpireTypeToString( gift.ExpireType ) ),
                DataAccess.CreateParameterDateTime( gift.FixedExpireDate ),
                DataAccess.CreateParameterInt32( gift.NumberOfDays ),
                DataAccess.CreateParameterBool( gift.IsElectronic )
            );

            return gift;
        }

        public GiftCertificateProduct Update( GiftCertificateProduct product )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE ProductGiftCertificate " +
                "SET GiftAmount = @GiftAmount, " +
                "ExpireType = @ExpireType, " +
                "FixedExpireDate = @FixedExpireDate, " +
                "NumberOfDays = @NumberOfDays, " +
                "IsElectronic = @IsElectronic " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterDecimal( product.GiftAmount ),
                DataAccess.CreateParameterString( product.ExpireType.ToString() ),
                DataAccess.CreateParameterDateTime( product.FixedExpireDate ),
                DataAccess.CreateParameterInt32( product.NumberOfDays ),
                DataAccess.CreateParameterBool( product.IsElectronic ),
                DataAccess.CreateParameterString( product.ProductID )
            );

            return product;
        }

        public void DeleteByProductID( string productID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductGiftCertificate " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterString( productID )
            );
        }

        public Hashtable GetDetails( string productID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllProductGiftCertificateFields +
                "FROM ProductGiftCertificate " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterString( productID )
            );

            if (table.Rows.Count > 0)
                return DataAccessConverter.ConvertDataRowToHashtable( table.Rows[0] );
            else
                return null;
        }
    }
}
