﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using Vevo.Shared.DataAccess;
using System.Data;
using Vevo.Domain.DataInterfaces;
using System.Collections;
using Vevo.Base.Data.Implementation;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data.Implementation
{
    internal class ShippingAddressAccess : BaseListedDataAccess<string, ShippingAddress>
    {
        #region Private

        private const string AllShippingAddressFields =
          " ShippingAddressID, CustomerID, AliasName, ShippingFirstName, " +
           "ShippingLastName, ShippingCompany, ShippingAddress1, ShippingAddress2, ShippingCity, " +
           "ShippingState, ShippingZip, ShippingCountry, ShippingPhone, ShippingFax, ShippingResidential, IsSameAsBillingAddress ";

        private ShippingAddress CreateOneFromRow( DataRow row )
        {
            ShippingAddress shippingAddress = new ShippingAddress( new Address(
                row["ShippingFirstName"].ToString(), row["ShippingLastName"].ToString(),
                row["ShippingCompany"].ToString(), row["ShippingAddress1"].ToString(),
                row["ShippingAddress2"].ToString(), row["ShippingCity"].ToString(),
                row["ShippingState"].ToString(), row["ShippingZip"].ToString(),
                row["ShippingCountry"].ToString(), row["ShippingPhone"].ToString(),
                row["ShippingFax"].ToString() ),
                ConvertUtilities.ToBoolean( row["ShippingResidential"] ) );

            shippingAddress.ShippingAddressID = row["ShippingAddressID"].ToString();
            shippingAddress.AliasName = row["AliasName"].ToString();
            shippingAddress.IsSameAsBillingAddress = ConvertUtilities.ToBoolean( row["IsSameAsBillingAddress"] );
            shippingAddress.CustomerID = row["CustomerID"].ToString();

            return shippingAddress;
        }

        private IList<ShippingAddress> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<ShippingAddress> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<ShippingAddress> list = new List<ShippingAddress>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }

        #endregion

        #region Protected

        protected override void Create( string customerID, ShippingAddress shippingAddress )
        {
            shippingAddress.ShippingAddressID = DataAccess.ExecuteScalar(
                "INSERT INTO ShippingAddress ( CustomerID, AliasName, ShippingFirstName, " +
                "ShippingLastName, ShippingCompany, ShippingAddress1, ShippingAddress2, ShippingCity, " +
                "ShippingState, ShippingZip, ShippingCountry, ShippingPhone, ShippingFax, ShippingResidential, IsSameAsBillingAddress ) " +
                "VALUES ( @CustomerID, @AliasName, @ShippingFirstName, " +
                "@ShippingLastName , @ShippingCompany, @ShippingAddress1, @ShippingAddress2, @ShippingCity, " +
                "@ShippingState, @ShippingZip, @ShippingCountry, @ShippingPhone, @ShippingFax, @ShippingResidential, @IsSameAsBillingAddress ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( customerID ),
                DataAccess.CreateParameterString( shippingAddress.AliasName ),
                DataAccess.CreateParameterString( shippingAddress.FirstName ),
                DataAccess.CreateParameterString( shippingAddress.LastName ),
                DataAccess.CreateParameterString( shippingAddress.Company ),
                DataAccess.CreateParameterString( shippingAddress.Address1 ),
                DataAccess.CreateParameterString( shippingAddress.Address2 ),
                DataAccess.CreateParameterString( shippingAddress.City ),
                DataAccess.CreateParameterString( shippingAddress.State ),
                DataAccess.CreateParameterString( shippingAddress.Zip ),
                DataAccess.CreateParameterString( shippingAddress.Country ),
                DataAccess.CreateParameterString( shippingAddress.Phone ),
                DataAccess.CreateParameterString( shippingAddress.Fax ),
                DataAccess.CreateParameterBool( shippingAddress.Residential ),
                DataAccess.CreateParameterBool( shippingAddress.IsSameAsBillingAddress )
                );

        }

        protected override void Update( string customerID, ShippingAddress shippingAddress )
        {
            DataAccess.ExecuteNonQuery(
              "UPDATE ShippingAddress " +
              "SET " +
              "    CustomerID = @CustomerID, " +
              "    AliasName = @AliasName, " +
              "    ShippingFirstName = @ShippingFirstName, ShippingLastName = @ShippingLastName, " +
              "    ShippingCompany = @ShippingCompany ,ShippingAddress1 = @ShippingAddress1, " +
              "    ShippingAddress2 = @ShippingAddress2, ShippingCity = @ShippingCity, " +
              "    ShippingState = @ShippingState, ShippingZip = @ShippingZip, " +
              "    ShippingCountry = @ShippingCountry, ShippingPhone = @ShippingPhone, " +
              "    ShippingFax = @ShippingFax, " +
              "    ShippingResidential = @ShippingResidential, " +
              "    IsSameAsBillingAddress = @IsSameAsBillingAddress " +
              "    WHERE ShippingAddressID = @ShippingAddressID;",

              DataAccess.CreateParameterString( customerID ),
              DataAccess.CreateParameterString( shippingAddress.AliasName ),
              DataAccess.CreateParameterString( shippingAddress.FirstName ),
              DataAccess.CreateParameterString( shippingAddress.LastName ),
              DataAccess.CreateParameterString( shippingAddress.Company ),
              DataAccess.CreateParameterString( shippingAddress.Address1 ),
              DataAccess.CreateParameterString( shippingAddress.Address2 ),
              DataAccess.CreateParameterString( shippingAddress.City ),
              DataAccess.CreateParameterString( shippingAddress.State ),
              DataAccess.CreateParameterString( shippingAddress.Zip ),
              DataAccess.CreateParameterString( shippingAddress.Country ),
              DataAccess.CreateParameterString( shippingAddress.Phone ),
              DataAccess.CreateParameterString( shippingAddress.Fax ),
              DataAccess.CreateParameterBool( shippingAddress.Residential ),
              DataAccess.CreateParameterBool( shippingAddress.IsSameAsBillingAddress ),
              DataAccess.CreateParameterString( shippingAddress.ShippingAddressID ) );
        }

        protected override void Delete( string customerID, ShippingAddress shippingAddress )
        {
            DataAccess.ExecuteNonQuery(
                  "DELETE " +
                  "FROM ShippingAddress " +
                  "WHERE ShippingAddressID = @ShippingAddressID; ",
                  DataAccess.CreateParameterString( shippingAddress.ShippingAddressID ) );
        }

        #endregion

        #region Public

        public IList<ShippingAddress> GetAll( string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT " + AllShippingAddressFields +
               "FROM ShippingAddress " +
               CreateSortByText( sortBy ),
               DataAccess.CreateParameterString( sortBy ) );

            return ConvertTableToList( table );
        }

        public IList<ShippingAddress> GetByCustomerID( string customerID )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT " + AllShippingAddressFields +
               "FROM ShippingAddress " +
               "WHERE CustomerID = @CustomerID; ",
               DataAccess.CreateParameterString( customerID ) );

            return ConvertTableToList( table );
        }

        public IList<TableSchemaItem> GetTableSchemas()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingAddressFields +
                "FROM ShippingAddress " +
                "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public IList<ShippingAddress> SearchShippingAddress(
           string customerID, string sortBy, SearchFilter searchfilter, int startIndex, int endIndex, out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( customerID ) );
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchfilter, searchfilter.FieldName, parameterList );

            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText( sortBy ) + ") as rownum, " +
                " * FROM (" +
                "SELECT " + AllShippingAddressFields +
                "FROM ShippingAddress " +
                commandSearch + " AND CustomerID = @CustomerID " +
                ")AS ShippingAddress";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery,
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );
            howManyItems = table.Rows.Count;

            return ConvertTableToList( table );
        }

        public void DeleteAllShippingAddress()
        {
            DataAccess.ExecuteNonQuery(
                  "DELETE " +
                  "FROM ShippingAddress " );
        }

        #endregion
    }
}
