using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Configuration;
using System.Web.Security;
using System.Text;
using System.Data.SqlClient;
using System.Configuration;
using Vevo.Shared.Domain.SystemServices;

namespace Vevo.Domain
{
    public class ConnectionString
    {
        #region Private

        private string CreateConnectionString(
            string serverName,
            string databaseName,
            string userName,
            string password,
            string tagname )
        {
            SqlConnectionStringBuilder builder = new SqlConnectionStringBuilder();
            builder.DataSource = serverName;
            builder.InitialCatalog = databaseName;

            if (userName != "" && password != "")
            {
                builder.UserID = userName;
                builder.Password = password;
            }
            else
                builder.IntegratedSecurity = true;

            return builder.ConnectionString;
        }

        private string CreateConnectionString(
            string fileName )
        {
            SqlConnectionStringBuilder builder = new SqlConnectionStringBuilder();
            builder.DataSource = fileName;
            return builder.ConnectionString;
        }

        #endregion

        #region Public Methods
        public string CreateConnectionStringAccess( string fileName )
        {
            return CreateConnectionString( fileName ) + ";Provider=Microsoft.Jet.OLEDB.4.0;";
        }

        public string CreateConnectionStringSQLServerAuthenticationText(
            string serverName,
            string databaseName,
            string userName,
            string password,
            string tagname )
        {
            return CreateConnectionString( serverName, databaseName, userName, password, tagname );
        }

        public string CreateConnectionStringWindowsAuthenticationText( 
            string serverName, string databaseName, string tagname )
        {
            return CreateConnectionString( serverName, databaseName, "", "", tagname );
        }

        public bool EncryptConnectionString( out string errMessage )
        {
            try
            {
                Configuration config = WebConfigurationManager.OpenWebConfiguration(
                    HttpContext.Current.Request.ApplicationPath );
                ConnectionStringsSection section
                    = (ConnectionStringsSection) config.GetSection( "connectionStrings" );
                if (section != null && !section.SectionInformation.IsProtected)
                {
                    //DataProtectionConfigurationProvider = DPAPI   RSAProtectedConfigurationProvider = RSA
                    section.SectionInformation.ProtectSection( "DataProtectionConfigurationProvider" );
                    config.Save( ConfigurationSaveMode.Modified );
                }
                errMessage = "";
                return true;
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                errMessage = ex.Message;
                return false;
            }
        }

        public bool WriteConnectionStringWithoutEncrypt(
            IFileManager fileManager,
            string connectionFilePath,
            string tagName,
            out string errMessage )
        {
            string connectionString = ConfigurationManager.ConnectionStrings[tagName].ConnectionString;
            string providerName = ConfigurationManager.ConnectionStrings[tagName].ProviderName;

            return WriteConnectionStringWithoutEncrypt(
                fileManager, connectionFilePath, connectionString, providerName, tagName, out errMessage );
        }

        public bool WriteConnectionStringWithoutEncrypt(
            IFileManager fileManager,
            string connectionFilePath,
            string connectionString,
            string providerName,
            string tagName,
            out string errMessage )
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine( "<connectionStrings>" );
            sb.AppendLine( "  <remove name=\""+tagName+"\"/>" );
            sb.AppendLine( "  <add name=\"" + tagName + "\" connectionString=\"" + connectionString + "\" providerName=\"" + providerName + "\"/>" );
            sb.AppendLine( "</connectionStrings>" );

            try
            {
                fileManager.SaveAll( connectionFilePath, sb.ToString() );
                errMessage = "";
                return true;
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                errMessage = ex.Message;
                return false;
            }
        }

        #endregion
    }
}
