﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using LumenWorks.Framework.IO.Csv;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.SystemServices;
using Vevo.Domain.Users;
using System.Web.Security;
using Vevo.Shared.Utilities;
using System.Collections;
using Vevo.Base.Domain;

namespace Vevo.Domain.ImportExport
{
    public class CustomerImporter
    {
        #region Private

        private const string _registerDate = "RegisterDate";
        private const string _userName = "UserName";
        private const string _firstName = "FirstName";
        private const string _lastName = "LastName";
        private const string _company = "Company";
        private const string _address1 = "Address1";
        private const string _address2 = "Address2";
        private const string _city = "City";
        private const string _state = "State";
        private const string _zip = "Zip";
        private const string _country = "Country";
        private const string _phone = "Phone";
        private const string _fax = "Fax";
        private const string _email = "Email";
        private const string _useBillingAsShipping = "UseBillingAsShipping";
        private const string _shippingFirstName = "ShippingFirstName";
        private const string _shippingLastName = "ShippingLastName";
        private const string _shippingCompany = "ShippingCompany";
        private const string _shippingAddress1 = "ShippingAddress1";
        private const string _shippingAddress2 = "ShippingAddress2";
        private const string _shippingCity = "ShippingCity";
        private const string _shippingState = "ShippingState";
        private const string _shippingZip = "ShippingZip";
        private const string _shippingCountry = "ShippingCountry";
        private const string _shippingPhone = "ShippingPhone";
        private const string _shippingFax = "ShippingFax";
        private const string _shippingResidential = "ShippingResidential";
        private const string _merchantNotes = "MerchantNotes";
        private const string _isWholesale = "IsWholesale";
        private const string _wholesaleLevel = "WholesaleLevel";
        private const string _affiliateCode = "AffiliateCode";
        private const string _affiliateExpireDate = "AffiliateExpireDate";
        private const string _isEnabled = "IsEnabled";
        private const string _isTaxExempt = "IsTaxExempt";
        private const string _qbListID = "QBListID";
        private const string _isQBExported = "IsQBExported";
        private const string _productIDsComparedList = "ProductIDsCompareList";
        private const string _taxExemptID = "TaxExemptID";
        private const string _taxExemptCountry = "TaxExemptCountry";
        private const string _taxExemptState = "TaxExemptState";
        private const string _fbUserID = "FBUserID";

        private const string _aliasName = "AliasName";
        private const string _isSameAsBillingAddress = "IsSameAsBillingAddress";

        private IFileManager _fileManager;
        private CustomerImportStatus _status;

        private DateTime _timeStart;
        private DateTime _timeEnd;


        private CsvReader CreateCsvReader( string relativePath, out TextReader reader )
        {
            Encoding encoding;
            if (_fileManager.ContainsUnicodeBom( relativePath ))
                encoding = Encoding.Unicode;
            else
                encoding = Encoding.Default;

            reader = _fileManager.CreateTextReader( relativePath, encoding );
            CsvReader csv = new CsvReader( reader, true );
            csv.MissingFieldAction = MissingFieldAction.ReplaceByEmpty;
            csv.ParseError += ParseErrorHandler;

            return csv;
        }

        private void ParseErrorHandler( object sender, ParseErrorEventArgs e )
        {
            _status.AddError();
        }

        private bool IsShippingAliasExist( CsvReader csv, Customer customer,out int shippingAddressIndex )
        {
            string aliasName = csv[_shippingFirstName] + " " + csv[_shippingLastName] + " " + csv[_shippingAddress1];

            System.Collections.ArrayList headerList = new System.Collections.ArrayList();
            headerList.AddRange( csv.GetFieldHeaders() );
            //for ShippingAddress import
            if (headerList.Contains( _aliasName ))
            {
                aliasName = csv[_aliasName];
            }

            IList<ShippingAddress> shippingAddressList = customer.ShippingAddresses;
            for (int i = 0; i < shippingAddressList.Count; i++)
            {
                ShippingAddress shippingAddress = shippingAddressList[i];
                if (shippingAddress.AliasName.Equals( aliasName ))
                {
                    shippingAddressIndex = i;
                    return true;
                }
            }
            shippingAddressIndex = -1;
            return false;
        }

        private Customer SetUpCustomer( CsvReader csv, Customer customer )
        {
            customer.RegisterDate = ConvertUtilities.ToDateTime( csv[_registerDate] );
            customer.UserName = csv[_userName];
            customer.BillingAddress = new Address(
                csv[_firstName],
                csv[_lastName],
                csv[_company],
                csv[_address1],
                csv[_address2],
                csv[_city],
                csv[_state],
                csv[_zip],
                csv[_country],
                csv[_phone],
                csv[_fax] );
            customer.Email = csv[_email];
            customer.UseBillingAsShipping = ConvertUtilities.ToBoolean( csv[_useBillingAsShipping] );

            ShippingAddress shippingAddress = new ShippingAddress( new Address(
                    csv[_shippingFirstName],
                    csv[_shippingLastName],
                    csv[_shippingCompany],
                    csv[_shippingAddress1],
                    csv[_shippingAddress2],
                    csv[_shippingCity],
                    csv[_shippingState],
                    csv[_shippingZip],
                    csv[_shippingCountry],
                    csv[_shippingPhone],
                    csv[_shippingFax] ),
                    ConvertUtilities.ToBoolean( csv[_shippingResidential] ) );

            shippingAddress.AliasName = csv[_shippingFirstName] + " " + csv[_shippingLastName] + " " + csv[_shippingAddress1];
            if (shippingAddress.AliasName.Length > 50)
                shippingAddress.AliasName = shippingAddress.AliasName.Substring( 0, 50 );

            shippingAddress.IsSameAsBillingAddress = ConvertUtilities.ToBoolean( csv[_useBillingAsShipping] );

            int shippingAddressIndex;
            if (!IsShippingAliasExist( csv, customer, out shippingAddressIndex ))
                customer.ShippingAddresses.Add( shippingAddress );

            customer.MerchantNotes = csv[_merchantNotes];
            customer.IsWholesale = ConvertUtilities.ToBoolean( csv[_isWholesale] );
            customer.WholesaleLevel = ConvertUtilities.ToInt32( csv[_wholesaleLevel] );
            customer.AffiliateCode = csv[_affiliateCode];
            customer.AffiliateExpireDate = ConvertUtilities.ToDateTime( csv[_affiliateExpireDate] );
            customer.IsEnabled = ConvertUtilities.ToBoolean( csv[_isEnabled] );
            customer.IsTaxExempt = ConvertUtilities.ToBoolean( csv[_isTaxExempt] );
            customer.QBListID = csv[_qbListID];
            customer.IsQBExported = ConvertUtilities.ToBoolean( csv[_isQBExported] );
            customer.ProductIDsCompareList = csv[_productIDsComparedList];
            customer.TaxExemptID = csv[_taxExemptID];
            customer.TaxExemptCountry = csv[_taxExemptCountry];
            customer.TaxExemptState = csv[_taxExemptState];
            customer.FBUserID = csv[_fbUserID];

            return customer;
        }

        private ShippingAddress SetUpShippingAddress( CsvReader csv )
        {
            ShippingAddress address = new ShippingAddress( new Address(
                    csv[_shippingFirstName],
                    csv[_shippingLastName],
                    csv[_shippingCompany],
                    csv[_shippingAddress1],
                    csv[_shippingAddress2],
                    csv[_shippingCity],
                    csv[_shippingState],
                    csv[_shippingZip],
                    csv[_shippingCountry],
                    csv[_shippingPhone],
                    csv[_shippingFax] ),
                    ConvertUtilities.ToBoolean( csv[_shippingResidential] ) );

            if (csv[_aliasName].Length > 50)
                address.AliasName = csv[_aliasName].Substring( 0, 50 );
            else
                address.AliasName = csv[_aliasName];
            address.CustomerID = DataAccessContext.CustomerRepository.GetIDFromUserName( csv[_userName] );
            address.IsSameAsBillingAddress = ConvertUtilities.ToBoolean( csv[_isSameAsBillingAddress] );
            return address;
        }

        private bool IsCustomerFieldsValid( CsvReader csv )
        {
            bool result = false;

            if (!string.IsNullOrEmpty( csv[_userName] ) &&
                !string.IsNullOrEmpty( csv[_firstName] ) &&
                !string.IsNullOrEmpty( csv[_lastName] ) &&
                !string.IsNullOrEmpty( csv[_country] ) &&
                !string.IsNullOrEmpty( csv[_address1] ) &&
                !string.IsNullOrEmpty( csv[_city] ) &&
                !string.IsNullOrEmpty( csv[_zip] ) &&
                !string.IsNullOrEmpty( csv[_email] ) &&
                !string.IsNullOrEmpty( csv[_shippingFirstName] ) &&
                !string.IsNullOrEmpty( csv[_shippingLastName] ) &&
                !string.IsNullOrEmpty( csv[_shippingCountry] ) &&
                !string.IsNullOrEmpty( csv[_shippingAddress1] ) &&
                !string.IsNullOrEmpty( csv[_shippingCity] ) &&
                !string.IsNullOrEmpty( csv[_shippingZip] ))
            {
                result = true;
            }

            return result;
        }

        private bool IsShippingAddressFieldsValid( CsvReader csv )
        {
            bool result = false;

            if (!String.IsNullOrEmpty( csv[_shippingFirstName] ) &&
                !String.IsNullOrEmpty( csv[_shippingLastName] ) &&
                !String.IsNullOrEmpty( csv[_shippingAddress1] ) &&
                !String.IsNullOrEmpty( csv[_shippingCity] ) &&
                !String.IsNullOrEmpty( csv[_shippingZip] ) &&
                !String.IsNullOrEmpty( csv[_shippingCountry] ) &&
                !String.IsNullOrEmpty( csv[_shippingResidential] ) &&
                !String.IsNullOrEmpty( csv[_aliasName] ) &&
                !String.IsNullOrEmpty( csv[_userName] ) &&
                !String.IsNullOrEmpty( csv[_isSameAsBillingAddress] ))
            {
                result = true;
            }

            return result;
        }

        private bool ImportCustomerValue( CsvReader csv )
        {
            bool result = false;

            try
            {
                Customer customer = new Customer();
                customer = SetUpCustomer( csv, customer );
                customer = DataAccessContext.CustomerRepository.Save( customer );

                Membership.CreateUser( csv[_userName].Trim(), csv[_zip].Trim(), csv[_email].Trim() );
                Roles.AddUserToRole( csv[_userName].Trim(), "Customers" );

                result = true;
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }

            return result;
        }

        private bool ImportShippingAddressValue( CsvReader csv, Customer customer )
        {
            bool result = false;

            try
            {
                ShippingAddress shippingAddress = SetUpShippingAddress( csv );
                customer.ShippingAddresses.Add( shippingAddress );
                DataAccessContext.CustomerRepository.Save( customer );
                result = true;
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }

            return result;
        }

        private bool UpdateShippingAddressValue( CsvReader csv, Customer customer,int shippingAddressIndex )
        {
            bool result = false;

            try
            {
                ShippingAddress shippingAddress = SetUpShippingAddress( csv );
                customer.ShippingAddresses.RemoveAt( shippingAddressIndex );
                customer.ShippingAddresses.Add(shippingAddress);
                DataAccessContext.CustomerRepository.Save( customer );
                result = true;
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }

            return result;
        }

        private CustomerImportStatus ImportCustomer( string filePath, out TimeSpan timeDiff )
        {
            TextReader reader = null;
            CsvReader csv = null;
            _status = new CustomerImportStatus();

            try
            {
                csv = CreateCsvReader( filePath, out reader );
                // Read data line-by-line
                int i = 2;
                while (csv.ReadNextRecord())
                {
                    if (IsCustomerFieldsValid( csv ))
                    {
                        Customer customer = DataAccessContext.CustomerRepository.GetOne( DataAccessContext.CustomerRepository.GetIDFromUserName( csv[_userName] ) );
                        if (!customer.IsNull)
                        {
                            customer = SetUpCustomer( csv, customer );
                            customer = DataAccessContext.CustomerRepository.Save( customer );
                            _status.UpdateSuccess();
                        }
                        else
                        {
                            try
                            {
                                if (ImportCustomerValue( csv ))
                                {
                                    _status.AddSuccess();
                                }
                            }
                            catch (Exception e)
                            {
                                _status.AddOtherError( i.ToString(), e );
                            }
                        }
                    }
                    else
                    {
                        _status.AddFieldError( i.ToString() );
                    }

                    i++;
                }

                _status.BuildOtherError();
                _status.BuildFieldError();
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }
            finally
            {
                csv.Dispose();
                reader.Dispose();
            }

            _timeEnd = DateTime.Now;
            timeDiff = _timeEnd.Subtract( _timeStart );
            return _status;
        }

        private CustomerImportStatus ImportShippingAddress( string filePath, out TimeSpan timeDiff )
        {
            TextReader reader = null;
            CsvReader csv = null;
            _status = new CustomerImportStatus();

            try
            {
                csv = CreateCsvReader( filePath, out reader );
                // Read data line-by-line
                int i = 2;
                while (csv.ReadNextRecord())
                {
                    if (IsShippingAddressFieldsValid( csv ))
                    {
                        Customer customer = DataAccessContext.CustomerRepository.GetOne( DataAccessContext.CustomerRepository.GetIDFromUserName( csv[_userName] ) );
                        if (customer.IsNull)
                        {
                            _status.AddCustomerNotFoundError( i.ToString() );
                        }
                        else
                        {
                            int shippingAddressIndex;
                            if (IsShippingAliasExist( csv, customer, out shippingAddressIndex ))
                            {
                                try
                                {
                                    if (UpdateShippingAddressValue( csv, customer, shippingAddressIndex ))
                                        _status.UpdateSuccess();
                                }
                                catch (Exception e)
                                {
                                    _status.AddOtherError( i.ToString(), e );
                                }
                            }
                            else
                            {
                                try
                                {
                                    if (ImportShippingAddressValue( csv, customer ))
                                        _status.AddSuccess();
                                }
                                catch (Exception e)
                                {
                                    _status.AddOtherError( i.ToString(), e );
                                }
                            }
                        }
                    }
                    else
                    {
                        _status.AddShippingFieldError( i.ToString() );
                    }

                    i++;
                }

                _status.BuildOtherError();
                _status.BuildCustomerNotFoundError();
                _status.BuildShippingFieldError();

            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }
            finally
            {
                csv.Dispose();
                reader.Dispose();
            }

            _timeEnd = DateTime.Now;
            timeDiff = _timeEnd.Subtract( _timeStart );
            return _status;
        }

        private void DeleteAllCustomer()
        {
            IList<Customer> customerList = DataAccessContext.CustomerRepository.GetAll( "CustomerID" );
            foreach (Customer customer in customerList)
            {
                DataAccessContext.CustomerRepository.Delete( customer.CustomerID );
                Membership.DeleteUser( customer.UserName );
            }
        }

        #endregion

        #region Constructors

        public CustomerImporter( IFileManager fileManager )
        {
            _fileManager = fileManager;
        }

        public CustomerImporter()
        {
            _fileManager = new FileManager();
        }

        #endregion

        #region Public Methods

        public CustomerImportStatus ImportCustomerPurgeAll( string filePath, out TimeSpan timeDiff )
        {
            _timeStart = DateTime.Now;
            DeleteAllCustomer();
            return ImportCustomer( filePath, out timeDiff );
        }

        public CustomerImportStatus ImportCustomerOverWrite( string filePath, out TimeSpan timeDiff )
        {
            _timeStart = DateTime.Now;
            return ImportCustomer( filePath, out timeDiff );
        }

        public CustomerImportStatus ImportShippingAddressPurgeAll( string filePath, out TimeSpan timeDiff )
        {
            _timeStart = DateTime.Now;
            DataAccessContext.CustomerRepository.DeleteAllShippingAddress();
            return ImportShippingAddress( filePath, out timeDiff );
        }

        public CustomerImportStatus ImportShippingAddressOverWrite( string filePath, out TimeSpan timeDiff )
        {
            _timeStart = DateTime.Now;
            return ImportShippingAddress( filePath, out timeDiff );
        }

        #endregion
    }
}
