using System;
using System.Collections.Generic;
using Vevo.Domain;
using Vevo.Domain.Products;
using Vevo.Domain.Stores;
using Vevo.QuickBooks.Qbxml70;
using Vevo.Shared.Utilities;

namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for ProductAddRequestState
    /// </summary>
    public class ProductAddRequestState : QBWCState
    {
        private void MoveToNextState()
        {
            QBWCSession.CurrentState = new ProductAddResponseState( QBWCSession );
        }

        private void MoveToOrderAddRqState()
        {
            QBWCSession.CurrentState = new OrderAddRequestState( QBWCSession );
        }

        private IList<Product> GetAddData()
        {
            string cultureID = DataAccessContext.CultureRepository.GetIDByName(
                DataAccessContext.Configurations.GetValue( "DefaultWebsiteLanguage" ) );
            Culture culture = DataAccessContext.CultureRepository.GetOne( cultureID );

            string storeID = getStoreID();
            string rootCategoryID = DataAccessContext.Configurations.GetValue( "0", "RootCategory", storeID );

            IList<Product> table = DataAccessContext.ProductRepository.GetByIsQBExported( 
                culture, false, rootCategoryID, storeID, String.Empty );
            
            return table;
        }

        private string getStoreID()
        {
            return DataAccessContext.Configurations.GetValue( "QBSelectStoreExport" );
        }

        private ItemNonInventoryAddRqType CreateOneRequest( Product product )
        {
            string productName = CreateUniqueName( QBUtilities.LimitLength( product.Name, QBConst.ProductNameMaxLength ) );

            ItemNonInventoryAddRqType req = new ItemNonInventoryAddRqType();
            req.ItemNonInventoryAdd = new ItemNonInventoryAdd();
            req.ItemNonInventoryAdd.Name = productName;

            SalesTaxCodeRef tax = new SalesTaxCodeRef();
            if (product.TaxClassID.Equals( "0" ))
                tax.FullName = "Non";
            else
                tax.FullName = "Tax";
            
            req.ItemNonInventoryAdd.SalesTaxCodeRef = tax;
            req.ItemNonInventoryAdd.ManufacturerPartNumber = product.ManufacturerPartNumber;

            SalesOrPurchase sales = new SalesOrPurchase();
            sales.Desc = QBUtilities.LimitLength( "ProductID: " + product.ProductID +
                " " + product.ShortDescription, QBConst.ProductDescMaxLength );
            sales.Item = ConvertUtilities.ToString( product.GetProductPrice( getStoreID() ).Price );
            sales.ItemElementName = ItemChoiceType2.Price;
            
            AccountRef accountRef = new AccountRef();
            accountRef.FullName = DataAccessContext.Configurations.GetValue( "QBIncomeAccountName" );
            sales.AccountRef = accountRef;
            req.ItemNonInventoryAdd.Item = sales;
            QBWCSession.AddQBFullNameList( productName, String.Empty );
            return req;
        }

        private string CreateProductAdd( out bool xmlReady )
        {
            IList<Product> table = GetAddData();
            if (table.Count == 0)
            {
                xmlReady = false;
                MoveToOrderAddRqState();
                return String.Empty;
            }
            // Write QBXML
            QBXML qbxml = CreateRequestQbxml();
            object[] requests = new object[table.Count];
            int index = 0;

            foreach (Product product in table)
            {
                requests[index] = CreateOneRequest( product );
                index++;
            }
            ((QBXMLMsgsRq) qbxml.Items[0]).Items = requests;

            xmlReady = true;
            string xml = CreateRequestXml( qbxml );
            return xml;
        }

        public ProductAddRequestState( QBWCSession session )
            : base( session )
        {
        }

        public override string OnSendRequestXml( out bool xmlReady )
        {
            MoveToNextState();

            return CreateProductAdd( out xmlReady );
        }
    }
}