﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Collections;
using Vevo.Shared.DataAccess;
using Vevo.Domain;

namespace Vevo.Reports
{
    public class StockReportBuilder
    {
        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }

        private DataTable GetStockReport(
            bool isLowStock, string cultureID)
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( cultureID ) );

            string script = "SELECT Product.ProductID, Product.Sku, ProductStock.Stock, " +
                            "ProductStock.OptionCombinationID, ProductLocale.Name " +
                            "FROM Product INNER JOIN " +
                            "ProductStock ON Product.ProductID = ProductStock.ProductID " +
                            "AND ProductStock.StockEnabled = 1 INNER JOIN " +
                            "ProductLocale ON Product.ProductID = ProductLocale.ProductID " +
                            "AND ProductLocale.CultureID = @CultureID AND Product.UseInventory = 1 ";
            if (isLowStock)
            {
                script += " AND ProductStock.Stock <= " + DataAccessContext.Configurations.GetIntValue( "OutOfStockValue" );
            }

            DataTable table = DataAccess.ExecuteSelect( script, DataAccess.ConvertArrayListToDbParameters( parameters ) );
            table.Columns.Add( "Option", typeof( string ) );
            foreach (DataRow row in table.Rows)
            {
                if (row["OptionCombinationID"].ToString() == "0")
                {
                    row["Option"] = "";
                }
                else
                {
                    string sql = "SELECT OptionCombination.OptionCombinationID, OptionItemLocale.Name, " +
                                "OptionItem.OptionItemID FROM OptionItemLocale INNER JOIN " +
                                "OptionItem ON OptionItemLocale.OptionItemID = OptionItem.OptionItemID INNER JOIN " +
                                "OptionCombination ON OptionItem.OptionItemID = OptionCombination.ProductOptionItemID " +
                                "AND OptionItemLocale.CultureID = @CultureID " +
                                "AND OptionCombination.OptionCombinationID = @OptionCombinationID ";
                    ArrayList SQLparameters = new ArrayList();
                    SQLparameters.Add( DataAccess.CreateParameterString( cultureID ) );
                    SQLparameters.Add( DataAccess.CreateParameterString( row["OptionCombinationID"].ToString() ) );
                    DataTable optionTable = DataAccess.ExecuteSelect( sql, 
                        DataAccess.ConvertArrayListToDbParameters( SQLparameters ) );
                    string optionName = String.Empty;
                    foreach (DataRow optionRow in optionTable.Rows)
                    {
                        optionName += " - " + optionRow["Name"];
                    }
                    optionName = optionName.TrimStart( " - ".ToCharArray() );
                    row["Option"] = optionName;
                }
            }

            return table;
        }

        public DataTable GetLowStockReport(
            string cultureID,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            DataTable full = GetStockReport( true, cultureID );
            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["ProductID"], full.Columns["OptionCombinationID"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        public DataTable GetLowStockReport(
            string cultureID,
            out int howManyItems )
        {
            DataTable full = GetStockReport( true, cultureID );
            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["ProductID"], full.Columns["OptionCombinationID"] };
            full.PrimaryKey = keys;
            return full;
        }

        public DataTable GetAllStockReport(
            string cultureID,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            DataTable full = GetStockReport( false, cultureID );
            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["ProductID"], full.Columns["OptionCombinationID"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        public DataTable GetAllStockReport(
            string cultureID,
            out int howManyItems )
        {
            DataTable full = GetStockReport( false, cultureID );
            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["ProductID"], full.Columns["OptionCombinationID"] };
            full.PrimaryKey = keys;
            return full;
        }
    }
}
